/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDAS
*       Filename : mysqldb.c
* 		Functions for performing MySQL db connections and result handling.
*
*       Updated :
*               11/10/2003 - Added function and header comments.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <mysql/mysql.h>

#include "../Commoninclude.h"

/******************************************************************************
*       struct DBR * dbQuery(char * query, struct SQLCFG * sqlcfg)
*		Connects to specified MySQL server, performs query, stores 
*		result and disconnects from server. Returns struct containing
*		number of results and results.
*
*	Args :
*		char * query		SQL Query string.
*		struct SQLCFG * sqlcfg	SQL Config struct.
*
*       Updated :
******************************************************************************/
struct DBR * dbQuery(char * query, struct SQLCFG * sqlcfg)
{
	MYSQL mysql;
	struct DBR * dbr = 0;
	
	mysql_init(&mysql);
	
	if(mysql_real_connect(&mysql, sqlcfg->IP, sqlcfg->user, sqlcfg->passwd, sqlcfg->db, sqlcfg->port, NULL, 0))
	{
        	if(!mysql_query(&mysql, query))
        	{
        		dbr = (struct DBR *) malloc(sizeof(struct DBR));
        		memset(dbr, 0, sizeof(struct DBR));
        		
    			if(dbr)
    			{
        			dbr->results = mysql_store_result(&mysql);
        	        
        	        	if(dbr->results)
        	        	{
        	        		dbr->num = mysql_num_rows(dbr->results);
        	        	} else {
        	        		dbr->num = 0;
        	        	}
        	        }
		}
		mysql_close(&mysql);
	}
	return(dbr);
}

/******************************************************************************
*       struct DBR * dbQuery_P(struct DBR * dbr, char * query)
*		Same functionality as dbQuery() but it assumes the DB 
*		conncetion exists and it does not close the connection on exit.
*
*	Args :
*		struct DBR * dbr	DB Result struct (contains connection
*					handle).
*		char * query            SQL Query string.
*
*       Updated :
******************************************************************************/
struct DBR * dbQuery_P(struct DBR * dbr, char * query)
{
	if(dbr)
	{
		if(&dbr->mysql)
		{
			if(!mysql_query(&dbr->mysql, query))
			{
				dbr->results = mysql_store_result(&dbr->mysql);
				if(dbr->results)
				{
					dbr->num = mysql_num_rows(dbr->results);
				} else {
					dbr->num = 0;
				}
			}
		}
	}
	
	return(dbr);
}

/******************************************************************************
*       struct DBR * dbConnect(struct SQLCFG * sqlcfg)
*		Connects to specified server. Returns DB Result which contains
*		MySQL connection handle.
*
*	Args :
*		struct SQLCFG * sqlcfg		SQL Config struct.
*
*       Updated :
******************************************************************************/
struct DBR * dbConnect(struct SQLCFG * sqlcfg)
{
	struct DBR * dbr = 0;
	
	dbr = (struct DBR *) malloc(sizeof(struct DBR));
	memset(dbr, 0, sizeof(struct DBR));
	
	mysql_real_connect(&dbr->mysql, sqlcfg->IP, sqlcfg->user, sqlcfg->passwd, sqlcfg->db, sqlcfg->port, NULL, 0);
	
	return(dbr);
}

/******************************************************************************
*       struct DBR * dbClose(struct DBR * dbr)
*		Closes MySQL connection.
*
*	Args :
*		struct DBR * dbr	SQL Config struct.
*
*       Updated :
******************************************************************************/
struct DBR * dbClose(struct DBR * dbr)
{
	mysql_close(&dbr->mysql);
	
	return(dbr);
}

/******************************************************************************
*       MYSQL_ROW dbResult(struct DBR * dbr)
*		Returns one row from DB Result.
*
*	Args :
*		struct DBR * dbr	DB Result struct.
*
*       Updated :
******************************************************************************/
MYSQL_ROW dbResult(struct DBR * dbr)
{
	MYSQL_ROW myrow;
	
	if(dbr)
	{
		if(dbr->num)
		{
			myrow = mysql_fetch_row(dbr->results);
		}
	}
	
	return(myrow);
}

/******************************************************************************
*       struct DBR * dbFree(struct DBR * dbr)
*		Frees both the DB Results and the Db result struct.
*
*	Args :
*		struct DBR * dbr	DB Result struct.
*
*       Updated :
******************************************************************************/	
struct DBR * dbFree(struct DBR * dbr)
{
	if(dbr)
	{
		if(dbr->num)
		{
    			mysql_free_result(dbr->results);
    		}
		free(dbr);
	}
	return(NULL);
}

/******************************************************************************
*       struct DBR * dbFree_P(struct DBR * dbr)
*		Frees Results from DB Result struct. Unlike dbFree() does not
*		free dbr struct itself.
*
*	Args :
*		struct DBR * dbr	DB Result struct.
*
*       Updated :
******************************************************************************/
struct DBR * dbFree_P(struct DBR * dbr)
{
	if(dbr)
	{
		if(dbr->num)
		{
    			mysql_free_result(dbr->results);
    		}
	}
	return(dbr);
}
