/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASd
*       Filename : MIDASd.c
*		Main functions for MIDASd (Data Collection Daemon).
*
*       Updated :
*               11/07/2003 - Added function and header comments.
*
******************************************************************************/
#include <mysql/mysql.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include <signal.h>
#include <wait.h>

#include "../Common/Commoninclude.h"

//
// MIDASd includes
#include "includes.h"

//
//	Global Var defined
//
char APPNAME[] = "MIDASd";
char CFGPATH[] = "/usr/local/MIDAS/etc/MIDASd.cf";
char * DEBUGMSG = 0;
char * KEYFILE = 0;
char STATUS[4][10] = {
		"RED", 
		"GREEN", 
		"YELLOW"
		};

struct SQLCFG * SQLCFG;
struct XMLLeaf * ConfigXMLTree = 0;

int DEBUGLEVEL = 0;

//
// Setting Logging Bits;
//
int	LOG_INFO	=1;	//General Info
int	LOG_ERROR	=2;	//Errors
int	LOG_SQL		=4;	//SQL Statements
int	LOG_SOCKET	=8;	//Socket related
int	LOG_CHECK	=16;	//Checks
int	LOG_FORK	=32;	//Fork Processes
int	LOG_XML		=4096;	//XML
int	LOG_ALWAYS	=32767;

/******************************************************************************
*       int main(int argc, char *argv[])
*		main() function for MIDASd
*
*	Args :
*		int argc	Argument count.
*		char *argv[]	Argument string array.
*
*       Updated :
******************************************************************************/
int main(int argc, char *argv[])
{
	(void) signal(SIGHUP, CatchHUP);
	
	(void) signal(SIGCHLD, FreeForked);

	ConfigXMLTree = ReadConfig(CFGPATH);
	if(!ConfigXMLTree)
	{
		printf("! Unable to open %s\n", CFGPATH);
		exit(0);
	}
	
	DEBUGLEVEL = Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DEBUGLEVEL"));
	KEYFILE = dstrcpy(KEYFILE, ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.KEYFILE"));
	
	if(!ParseCommandLine(argc, argv))
	{
		exit(0);
	}
	
	SQLCFG = (struct SQLCFG *) malloc(sizeof(struct SQLCFG));
	bzero(SQLCFG, sizeof(struct SQLCFG));
	
	SQLCFG->IP = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SQL.IP");
	SQLCFG->user = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SQL.USER");
	SQLCFG->passwd = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SQL.PASSWD");
	SQLCFG->db = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SQL.DB");
	SQLCFG->port = Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SQL.port")); 
	
	
	if(Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DAEMON")))
	{
		if(fork())
		{
			exit(0);
		}
	}
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[main]-[MIDASd %s starting]", MIDASVER);
	
	StartServer();
	
	DelXMLLeafTree(ConfigXMLTree);
	dfree((char *)SQLCFG);
	
	exit(0);

}

/******************************************************************************
*       struct XMLLeaf * ReadConfig(char * filename)
*		Reads XML config file and returns an XML linked list of config
*		options.
*
*	Args :
*		char * filename		Path and filename of XML config file
*					to read.
*
*       Updated :
******************************************************************************/
struct XMLLeaf * ReadConfig(char * filename)
{
	FILE *fp = 0;
	struct XMLLeaf * LocalLeaf = 0;
	char * FileString = 0;
		
	fp = fopen(filename, "r");
	if(fp)
	{
		FileString = dfiletostring(fp);
		if(FileString)
		{
			LocalLeaf = StringToXMLTree(FileString);
		}
		FileString = dfree(FileString);
		fclose(fp);
	}
	return(LocalLeaf);	
}		

/******************************************************************************
*       void CatchHUP(int sig)
*		Function triggered on SIGHUP signal and causes the config 
*		file to be re read.
*
*	Args :
*		int sig		Signal that caused trigger.
*
*       Updated :
******************************************************************************/
void CatchHUP(int sig)
{
	MyDebug(__LINE__, __FILE__,  LOG_INFO, "[CatchHUP]-[Reloading config file : %s]", CFGPATH);
 
	ConfigXMLTree = ReadConfig(CFGPATH);
}

/******************************************************************************
*       void FreeForked(int sig)
*		Function triggered on SIGCHLD (Endof life signal from Child
*		process). Perfoms cleanup so child process can go away.
*
*	Args :
*		int sig		Signal that caused trigger.
*
*       Updated :
******************************************************************************/
void FreeForked(int sig)
{
	pid_t pid = 1;
	
	while(pid > 0)
	{
		pid = waitpid(0, NULL, WNOHANG);
	}

}

/******************************************************************************
*       int ParseCommandLine(int argc, char *argv[])
*		Parses command line arguments.
*
*	Args :
*		int argc	Argument count.
*		char *argv[]	Argument string array.
*
*       Updated :
******************************************************************************/
int ParseCommandLine(int argc, char *argv[])
{
	int opt = 0;
	int Status = 1;
	char *Key = 0;
	FILE *fp;
		
	while(opt != -1)
	{
		opt = getopt(argc, argv, "KVhTD:");
		switch(opt)
		{
			case 'K':
				Key = MakeKey();
				fp = fopen(KEYFILE, "w");
				if(fp)
				{
					fprintf(fp, "%s", Key);
					printf("%s %s\n", APPNAME, MIDASVER);
					printf("New key generated in %s.\n", KEYFILE);
					fclose(fp);
				} else {
					printf(" ! Unable to open %s\n", KEYFILE);
				}
				Status = 0;
				break;
			case 'V':
				printf("%s %s\n", APPNAME, MIDASVER);
				Status = 0;
				break;
			case 'h':
				printf("%s %s\n", APPNAME, MIDASVER);
				printf("\t-D#\tSet Debug level\n");
				printf("\t-h\tShow (this) help screen\n");
				printf("\t-K\tBuild keyfile (%s)\n", KEYFILE);
				printf("\t-T\tShow Configuration\n");
				printf("\t-V\tShow version\n");
				
				
				Status = 0;
				break;
			case 'T':
				ShowConfig();
				Status = 0;
				break;
			case 'D':
				DEBUGLEVEL = Myatoi(optarg);
				break;
		}
	}
	return(Status);
}

/******************************************************************************
*       void ShowConfig(void)
*		Will display the current configuration settings. This is 
*		triggered witht he -T command line option.
*
*
*       Updated :
******************************************************************************/
void ShowConfig()
{
        printf("%s %s\n", APPNAME, MIDASVER);
	printf("------------------------------------------------------------------\n");
        printf("[Contents of %s.cf file]\n", APPNAME); 
        DebugXMLTree("MIDASd.cf", ConfigXMLTree, 0);
}
